'use strict';

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);
  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    enumerableOnly && (symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    })), keys.push.apply(keys, symbols);
  }
  return keys;
}
function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = null != arguments[i] ? arguments[i] : {};
    i % 2 ? ownKeys(Object(source), !0).forEach(function (key) {
      _defineProperty(target, key, source[key]);
    }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) {
      Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
    });
  }
  return target;
}
function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}
function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, _toPropertyKey(descriptor.key), descriptor);
  }
}
function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  Object.defineProperty(Constructor, "prototype", {
    writable: false
  });
  return Constructor;
}
function _defineProperty(obj, key, value) {
  key = _toPropertyKey(key);
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }
  return obj;
}
function _toPrimitive(input, hint) {
  if (typeof input !== "object" || input === null) return input;
  var prim = input[Symbol.toPrimitive];
  if (prim !== undefined) {
    var res = prim.call(input, hint || "default");
    if (typeof res !== "object") return res;
    throw new TypeError("@@toPrimitive must return a primitive value.");
  }
  return (hint === "string" ? String : Number)(input);
}
function _toPropertyKey(arg) {
  var key = _toPrimitive(arg, "string");
  return typeof key === "symbol" ? key : String(key);
}

/* eslint-disable array-callback-return */
/* eslint-disable no-underscore-dangle */
/** клиентская часть плагина роутера, позволяет
 * аторизовать с помощью логин/пароль или идентификатора сессии sid
 *
 * Ex:
 * import router from 'fmihel-php-router-client';
 * import Session from 'fmihel-php-session-client';
 * let session = new Session(router);
 *
 * session.on('autprize',()=>{
 *     console.log('after autorize');
 * })
 * session.on('logout',()=>{
 *     console.log('after logout');
 * })
 *
 * session.autorize({pass:'xxx',login:'name'});
 *
 * session.logout();
 *
 */
var Session = /*#__PURE__*/function () {
  function Session(router) {
    _classCallCheck(this, Session);
    this["private"] = {
      enabled: false,
      data: {} // информация о сессии
    };

    this.router = router;
    this.before = this.before.bind(this);
    this.after = this.after.bind(this);
    this.events = {
      autorize: [],
      logout: []
    };
    this.paths = ['session/autorize', 'session/logout'];
  }

  /** обработчик перед отправкой,
   * в отправляемый пакет добавляет информацию о сесси
  */
  _createClass(Session, [{
    key: "before",
    value: function before(pack) {
      return _objectSpread2(_objectSpread2({}, pack), {}, {
        session: this["private"].data
      });
    }

    /** обработчик сразу, как приходит информация
     * в отправляемый пакет добавляет информацию о сесси
    */
  }, {
    key: "after",
    value: function after(pack) {
      var to = pack.to,
        session = pack.session;
      // если это не информация об авторизации и пакет pack.session === [] то сервер не подтвердил авторизацию
      // и значит не производил обработку входящег пакета, и значит разрываем авторизацию
      if (this.paths.indexOf(to) < 0 && Array.isArray(session) && session.length === 0) {
        this._close();
      }
      return pack;
    }
  }, {
    key: "_close",
    value: function _close() {
      if (this["private"].enabled) {
        this["private"].enabled = false;
        this["private"].data = {};
        this["do"]('logout');
        return true;
      }
      return false;
    }

    /** признак, авторизованы или нет */
  }, {
    key: "enabled",
    value: function enabled() {
      return this["private"].enabled;
    }

    /** запрос на авторизацию, будет обработан плагином php-session
     * можно отправить или login и pass или sid
    */
  }, {
    key: "autorize",
    value: function autorize(_ref) {
      var login = _ref.login,
        pass = _ref.pass,
        sid = _ref.sid;
      var t = this;
      return t.router.send({
        to: 'session/autorize',
        data: _objectSpread2(_objectSpread2(_objectSpread2({}, login ? {
          login: login
        } : {}), pass ? {
          pass: pass
        } : {}), sid ? {
          sid: sid
        } : {})
      }).then(function (data) {
        if ('sid' in data) {
          var prev = _objectSpread2(_objectSpread2({}, t["private"]), {}, {
            data: _objectSpread2({}, t["private"].data)
          });
          t["private"].enabled = true;
          t["private"].data = _objectSpread2({}, data);
          if (!prev.enabled || prev.data.sid !== data.sid) t["do"]('autorize');
        } else {
          t._close();
          throw new Error('data is empty');
        }
        return data;
      })["catch"](function (e) {
        t._close();
        throw new Error(e);
      });
    }

    /** разрыв авторизации */
  }, {
    key: "logout",
    value: function logout() {
      if (this._close()) {
        this.router.send({
          to: 'session/logout'
        });
      }
    }

    /** добавляет события autorize logout , возвращает метод отмены события */
  }, {
    key: "on",
    value: function on(event, callback) {
      var t = this;
      if (!(event in t.events)) throw Error(" event=\"".concat(event, "\" not in ").concat(this.events.join(',')));
      t.events[event].push(callback);
      return function () {
        t.events[event] = t.events[event].filter(function (cb) {
          return cb !== callback;
        });
      };
    }

    /** выполняет все сохраненные ф-ции соотвествующие событию */
  }, {
    key: "do",
    value: function _do(event) {
      var param = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var t = this;
      t.events[event].map(function (callback) {
        callback({
          param: param,
          sender: t,
          event: event
        });
      });
    }
  }]);
  return Session;
}();

module.exports = Session;
